#include <stdio.h>
#include <stdint.h>
#include <fcntl.h>

#include <stdlib.h>

#include "servo.h"

#define NUM_LEGS 6

// base geometry
const float x_offsets[NUM_LEGS]={ 55.0f, 0,-55.0f,-55.0f, 0, 55.0f}; // X-Y coords center of coxa, mm
const float y_offsets[NUM_LEGS]={ 29.0f, 42.0f, 29.0f,-29.0f,-42.0f,-29.0f}; // X is line from back to front ; Y is line from left to right if see from back
//				 l/frnt, l/mid,l/back,r/back, r/mid,r/frnt

int8_t virt_servo_direction[NUM_SERVOS]={
 1, -1, -1,
 1, -1, -1,
 1, -1, -1,
 1, 1, 1,
 1, 1, 1,
 1, 1, 1,
};


float phi1[6];
float phi2[6];
float phi3[6];

void
draw_hexapod() {
   const float c =13.0f; // length coxa , mm
   const float f =51.0f; // length femur, mm
   const float t =80.0f; // length tibia, mm
   const float p1=10.0f; // knee thinkness

   const float thickness=5.0f;

   int i,fd;

   fd=open("/tmp/servodata",O_RDONLY);
   if(fd==-1) {
        perror("/tmp/servodata");
        exit (20);
   } else {
      read(fd,servo_angle,sizeof(servo_angle));
      close(fd);
      system ("clear");
      for(i=0;i<6;i++) {
         phi1[i]=servo_angle[i*3];
         phi2[i]=servo_angle[i*3+1];
         phi3[i]=servo_angle[i*3+2];
         phi1[i]*=virt_servo_direction[i*3];
         phi2[i]*=virt_servo_direction[i*3+1];
         phi3[i]*=virt_servo_direction[i*3+2];

      printf("   c[%d]=%.1f ; f[%d]=%.1f ; t[%d]=%.1f \n",i,phi1[i],i,phi2[i],i,phi3[i]);
      }
   }


   // base plane
   glRotatef(-70,1,0,1);

   // body color
   glColor3f(0.8,0.8,0.8);

   // body right and left triangles
   glBegin(GL_TRIANGLES);
      for(i=0;i<NUM_LEGS;i++) {
         glVertex3f(x_offsets[i]/1000,y_offsets[i]/1000,0);
      }
   glEnd();

   // rectangle body mid plane
   glBegin(GL_QUADS);
      glVertex3f(x_offsets[0]/1000,y_offsets[0]/1000,0);
      glVertex3f(x_offsets[2]/1000,y_offsets[2]/1000,0);
      glVertex3f(x_offsets[3]/1000,y_offsets[3]/1000,0);
      glVertex3f(x_offsets[5]/1000,y_offsets[5]/1000,0);
   glEnd();

   for(i=0;i<NUM_LEGS;i++) {
      glPushMatrix();

      // move into leg base position
      glTranslatef(x_offsets[i]/1000,y_offsets[i]/1000,0);
      // rotate 180 degrees if on other side
      if(i>=(NUM_LEGS/2)) {
         glRotatef(-180,0,0,1);
      }

      // Coxa
      glRotatef(phi1[i],0,0,1);
      glColor3f(1,1,0);
      glBegin(GL_QUADS);
         glVertex3f(0,0,0);
         glVertex3f(0,c/1000,0);
         glVertex3f(thickness/1000,c/1000,0);
         glVertex3f(thickness/1000,0,0);
      glEnd();

      // Femur
      glTranslatef(0,c/1000,0);
      glRotatef(90,0,1,0);
      glRotatef(phi2[i],0,0,1);
      glColor3f(0,0,1);
      glBegin(GL_QUADS);
         glVertex3f(0,0,0);
         glVertex3f(0,f/1000,0);
         glVertex3f(thickness/1000,f/1000,0);
         glVertex3f(thickness/1000,0,0);

         glVertex3f(0,f/1000,0);
         glVertex3f(0,0,0);
         glVertex3f(0,0,thickness/1000);
         glVertex3f(0,f/1000,thickness/1000);

         glVertex3f(0,f/1000,0);
         glVertex3f(thickness/1000,f/1000,0);
         glVertex3f(0,f/1000,thickness/1000);
         glVertex3f(thickness/1000,f/1000,thickness/1000);

         glVertex3f(thickness/1000,f/1000,0);
         glVertex3f(thickness/1000,0,0);
         glVertex3f(thickness/1000,0,thickness/1000);
         glVertex3f(thickness/1000,f/1000,thickness/1000);


         glVertex3f(0,0,thickness/1000);
         glVertex3f(0,f/1000,thickness/1000);
         glVertex3f(thickness/1000,f/1000,thickness/1000);
         glVertex3f(thickness/1000,0,thickness/1000);
      glEnd();

      glTranslatef(0,f/1000,0);

      glRotatef(-90-phi3[i],0,0,1);
      glColor3f(1,0,0);
      glBegin(GL_QUADS);
         glVertex3f(  p1/1000, 0, thickness/2/1000);
         glVertex3f(0,  -p1/1000, thickness/2/1000);
         glVertex3f( -p1/1000, 0, thickness/2/1000);
         glVertex3f(0,    t/1000, thickness/2/1000);

         glVertex3f(  p1/1000, 0, thickness/2/1000);
         glVertex3f(  p1/1000, 0, -thickness/2/1000);
         glVertex3f(0,  -p1/1000, thickness/2/1000);
         glVertex3f(0,  -p1/1000, -thickness/2/1000);

         glVertex3f(0,  -p1/1000, thickness/2/1000);
         glVertex3f(0,  -p1/1000, -thickness/2/1000);
         glVertex3f( -p1/1000, 0, thickness/2/1000);
         glVertex3f( -p1/1000, 0, -thickness/2/1000);

         glVertex3f( -p1/1000, 0, thickness/2/1000);
         glVertex3f( -p1/1000, 0, -thickness/2/1000);
         glVertex3f(0,    t/1000, thickness/2/1000);
         glVertex3f(0,    t/1000, -thickness/2/1000);

         glVertex3f(0,    t/1000, thickness/2/1000);
         glVertex3f(  p1/1000, 0, -thickness/2/1000);
         glVertex3f(  p1/1000, 0, thickness/2/1000);
         glVertex3f(0,    t/1000, -thickness/2/1000);


         glVertex3f(  p1/1000, 0, -thickness/2/1000);
         glVertex3f(0,  -p1/1000, -thickness/2/1000);
         glVertex3f( -p1/1000, 0, -thickness/2/1000);
         glVertex3f(0,    t/1000, -thickness/2/1000);
      glEnd();
      glPopMatrix();
   }

}