#include "servo.h"

#include <string.h>
#include <stdio.h>
#include <fcntl.h>

#define NUM_SERVOS 18

// target servo angle
float servo_angle[NUM_SERVOS]={};
// target pwm servo value
uint16_t servo_pwm[NUM_SERVOS]={};

// current servo angle
float servo_current_angle[NUM_SERVOS]={};

// current pwm servo value
uint16_t servo_current_pwm[NUM_SERVOS]={};

// servo port numbering
uint8_t servo_port[NUM_SERVOS]={
 24, 25, 26, // left front
 20, 21, 22, // left mid
 16, 17, 18, // left bottom
 0, 1, 2, // right bottom
 4, 5, 6, // right mid
 8, 9, 10, // right front
};

// servo direction
int8_t servo_direction[NUM_SERVOS]={
 -1, 1, 1, // left front
 -1, 1, 1, // left mid
 -1, 1, 1, // left bottom
 -1, 1, 1, // right bottom
 -1, 1, 1, // right mid
 -1, 1, 1, // right front
};


/* Servo zerro offset pwm value */

uint16_t servo_zero_pwm[NUM_SERVOS]={
 1548, 1403, 1373,
 1457, 1476, 1431,
 1472, 1448, 1389,
 1449, 1455, 1476,
 1509, 1530, 1542,
 1475, 1489, 1417,
};

/* Servo min pwm value */

uint16_t servo_min_pwm[NUM_SERVOS]={
 2500,  520,  593,
 2237,  536,  681,
 1972,  638,  619,
 1019, 2315, 2316,
  759, 2496, 2382,
  535, 2449, 2287,
};

/* Servo max pwm value */

uint16_t servo_max_pwm[NUM_SERVOS]={
 1168, 2403, 2423,
  757, 2500, 2421,
  552, 2408, 2459,
 2459,  575,  546,
 2199,  576,  622,
 1865,  529,  547,
};

/* Servo min angle value */

float servo_min_angle[NUM_SERVOS]={
 -85.0f, -95.0f, -70.0f,
 -60.0f, -90.0f, -70.0f,
 -50.0f, -80.0f, -60.0f,
 50.0f, 80.0f, 60.0f,
 60.0f, 90.0f, 70.0f,
 85.0f, 95.0f, 70.0f,
};

/* Servo max angle value */

float servo_max_angle[NUM_SERVOS]={
 50.0f, 90.0f, 90.0f,
 70.0f, 90.0f, 90.0f,
 85.0f, 90.0f, 90.0f,
 -85.0f, -90.0f, -90.0f,
 -70.0f, -90.0f, -90.0f,
 -50.0f, -90.0f, -90.0f,
};

/* Servo angle to pwm factor */

float servo_factor[NUM_SERVOS]={
 -9.87f, 10.18f, 11.44f,
 -11.38f, 10.91f, 10.88f,
 -10.52f, 10.41f, 12.27f,
 -10.67f, 10.24f, 11.80f,
 -11.08f, 10.67f, 11.00f,
 -9.85f, 10.38f, 10.88f,
};



void
load_calibration(char *filename) {
   float angle_buffer[NUM_SERVOS];
   uint16_t pwm_buffer[NUM_SERVOS];
   int bytes_read;
   int fd;
   fd=open(filename,O_RDONLY);
   if(fd!=-1) {

      // servo_zero_pwm
      bytes_read=read(fd,pwm_buffer,sizeof(pwm_buffer));
      if(bytes_read==sizeof(pwm_buffer)) {
         memcpy(servo_zero_pwm,pwm_buffer,sizeof(pwm_buffer));
      } else {
         printf("load_calibration(%s),zero_pwm zone : short read\n",filename);
      }

      // servo_min_pwm
      bytes_read=read(fd,pwm_buffer,sizeof(pwm_buffer));
      if(bytes_read==sizeof(pwm_buffer)) {
         memcpy(servo_min_pwm,pwm_buffer,sizeof(pwm_buffer));
      } else {
         printf("load_calibration(%s),min_pwm zone: short read\n",filename);
      }

      // servo_max_pwm
      bytes_read=read(fd,pwm_buffer,sizeof(pwm_buffer));
      if(bytes_read==sizeof(pwm_buffer)) {
         memcpy(servo_max_pwm,pwm_buffer,sizeof(pwm_buffer));
      } else {
         printf("load_calibration(%s),max_pwm zone: short read\n",filename);
      }

      // servo_min_angle
      bytes_read=read(fd,angle_buffer,sizeof(angle_buffer));
      if(bytes_read==sizeof(angle_buffer)) {
         memcpy(servo_min_angle,angle_buffer,sizeof(angle_buffer));
      } else {
         printf("load_calibration(%s),min_angle zone: short read\n",filename);
      }

      // servo_max_angle
      bytes_read=read(fd,angle_buffer,sizeof(angle_buffer));
      if(bytes_read==sizeof(angle_buffer)) {
         memcpy(servo_max_angle,angle_buffer,sizeof(angle_buffer));
      } else {
         printf("load_calibration(%s),max_angle zone: short read\n",filename);
      }

      // servo_factor
      bytes_read=read(fd,angle_buffer,sizeof(angle_buffer));
      if(bytes_read==sizeof(angle_buffer)) {
         memcpy(servo_factor,angle_buffer,sizeof(angle_buffer));
      } else {
         printf("load_calibration(%s),factor zone: short read\n",filename);
      }

      close(fd);
   } else {
      perror(filename);
   }
}
