#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <stdio.h>

#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>

#include "ssc32.h"
#include "servo.h"

static struct ssc32_message ssc32_message;
static struct ssc32_result  ssc32_result;

static int  ssc32_fd , ssc32_fake_fd;


void
comm_open(char ssc32_device[255]) {
//   ssc32_fd=open("/dev/tty.BT-ECU2-S-DevB",O_RDWR);
   ssc32_fd=open(ssc32_device,O_RDWR);
   if(ssc32_fd==-1) {
      perror("Error ssc32 device conection");
      exit(20);
   }
   ssc32_fake_fd=open("/tmp/servodata",O_WRONLY|O_CREAT|O_TRUNC,0700);

}

void
comm_close() {
   close(ssc32_fd);
   close(ssc32_fake_fd);
}

void

comm_update_servos(uint16_t time) {
   int i,n;

   // update fakeserv info
   if(ssc32_fake_fd!=-1) {
      lseek (ssc32_fake_fd, 0L, SEEK_SET);
      write(ssc32_fake_fd,servo_angle,sizeof(servo_angle));
   }
/*
   //query status
   sprintf (ssc32_message.cmd,"Q\n");
   write(ssc32_fd,&ssc32_message.cmd,2);
   read(ssc32_fd,ssc32_result.data,1);
//   printf("A:%x\n",ssc32_result.data[0]);
   while (ssc32_result.data[0] == SSC32_BUSY){
      printf("Wait controller\n");
      write(ssc32_fd,&ssc32_message.cmd,2);
      read(ssc32_fd,ssc32_result.data,1);
   }
*/
     ssc32_message.length=0;
     n=0;
     for(i=0;i<NUM_SERVOS;i++) {
        if (servo_angle[i]!=servo_current_angle[i]) {
           // calculate pwm from angle
           servo_pwm[i]=servo_angle[i]*servo_factor[i]+servo_zero_pwm[i];
           // stop in max and min positions
//           if (servo_pwm[i]>servo_max_pwm[i]) servo_pwm[i]=servo_max_pwm[i];
//           if (servo_pwm[i]<servo_min_pwm[i]) servo_pwm[i]=servo_min_pwm[i];
           sprintf (ssc32_message.cmd+8*n,"%c%02d%c%04d",CMD_SET_SERVOS,servo_port[i],CMD_SET_SERVOS_PWM,servo_pwm[i]);
           servo_current_angle[i]=servo_angle[i];
//           printf ("Update servo %d\n",i);
           ssc32_message.length+=8;
           n++;
        }
     }

     //send command
     if (ssc32_message.length > 0 ) {
        sprintf (ssc32_message.cmd+8*n,"T%04d\n",time);ssc32_message.length+=6;
        write(ssc32_fd,&ssc32_message.cmd,ssc32_message.length);
//        printf(ssc32_message.cmd);
     }
}


comm_update_servos_pwm(uint16_t time) {
   int i,n;

   //query status
   sprintf (ssc32_message.cmd,"Q\n");
   write(ssc32_fd,&ssc32_message.cmd,2);
   read(ssc32_fd,ssc32_result.data,1);
//   printf("A:%x\n",ssc32_result.data[0]);
   while (ssc32_result.data[0] == SSC32_BUSY){
      printf("Wait controller\n");
      write(ssc32_fd,&ssc32_message.cmd,2);
      read(ssc32_fd,ssc32_result.data,1);
   }

     ssc32_message.length=0;
     n=0;
     for(i=0;i<NUM_SERVOS;i++) {
        if (servo_pwm[i]!=servo_current_pwm[i]) {
           sprintf (ssc32_message.cmd+8*n,"%c%02d%c%04d",CMD_SET_SERVOS,servo_port[i],CMD_SET_SERVOS_PWM,servo_pwm[i]);
           servo_current_pwm[i]=servo_pwm[i];
//           printf ("Update servo %d\n",i);
           ssc32_message.length+=8;
           n++;
        }
     }

     //send command
     if (ssc32_message.length > 0 ) {
        sprintf (ssc32_message.cmd+8*n,"T%04d\n",time);ssc32_message.length+=6;
        write(ssc32_fd,&ssc32_message.cmd,ssc32_message.length);
//        printf(ssc32_message.cmd);
     }
}


void
comm_set_led(uint8_t led) {
/*
   int i;
   spi_result.status=SPI_BUSY; // assume busy
   while(spi_result.status) {
      if((spi_result.status==SPI_ERR)||(spi_result.status=SPI_DONE)) {
         spi_message.cmd=CMD_FINISH;
         write(spi_fd,&spi_message,1);
      } else if((spi_result.status==SPI_WAIT)) {
         spi_message.cmd=CMD_CONTINUE;
         write(spi_fd,&spi_message,1);
      }
      read(spi_fd,&spi_result,1);
   }

   spi_message.cmd=CMD_SET_LED;
   spi_message.length=1;
   spi_message.data[0]=led;
   
   spi_message.checksum=0x00;
   for(i=0;i<spi_message.length;i++) {
      spi_message.checksum^=spi_message.data[i];
   }
   write(spi_fd,&spi_message,sizeof(spi_message));

   while(spi_result.status!=SPI_WAIT) read(spi_fd,&spi_result,1);
   spi_message.cmd=CMD_CONTINUE;
   write(spi_fd,&spi_message,1);
   read(spi_fd,&spi_result,1);
   while(spi_result.status==SPI_BUSY) read(spi_fd,&spi_result,1);
*/ 
}

void
comm_set_laser(uint8_t led) {
/*   int i;
   spi_result.status=SPI_BUSY; // assume busy
   while(spi_result.status) {
      if((spi_result.status==SPI_ERR)||(spi_result.status=SPI_DONE)) {
         spi_message.cmd=CMD_FINISH;
         write(spi_fd,&spi_message,1);
      } else if((spi_result.status==SPI_WAIT)) {
         spi_message.cmd=CMD_CONTINUE;
         write(spi_fd,&spi_message,1);
      }
      read(spi_fd,&spi_result,1);
   }

   spi_message.cmd=CMD_SET_LASER;
   spi_message.length=1;
   spi_message.data[0]=led;
   
   spi_message.checksum=0x00;
   for(i=0;i<spi_message.length;i++) {
      spi_message.checksum^=spi_message.data[i];
   }
   write(spi_fd,&spi_message,sizeof(spi_message));

   while(spi_result.status!=SPI_WAIT) read(spi_fd,&spi_result,1);
   spi_message.cmd=CMD_CONTINUE;
   write(spi_fd,&spi_message,1);
   read(spi_fd,&spi_result,1);
   while(spi_result.status==SPI_BUSY) read(spi_fd,&spi_result,1);
*/
}
