#include <unistd.h>
#include <math.h>
#include <stdio.h>
#include <fcntl.h>
#include <termios.h>

#include <sys/time.h>

#include "servo.h"
#include "ssc32.h"

#include "ik.h"
#include "dynamic_sequencer.h"

void ik_to_servos(bot *);
int get_time();

static bot idle_position = {
   {0,0,-150},  // body position
   {0,0,0},     // body rotation

   { // leg positions
      {{ 100, 70,   0},}, // leg 0
      {{   0, 100,   0},}, // leg 1
      {{-100, 70,   0},}, // ...
      {{-100,-70,   0},},
      {{   0,-100,   0},},
      {{ 100, -70,   0},}
   }
};


int
main(int argc, char **argv) {
   struct termios t;
   int flags_stdio;

   vector3d startup_vector={0,0,100};
   vector3d v;

   sequencer_walk_parameters wp;

   int quit=0;
   int frame=0;
   int t_frame_start, t_frame_end;
   char c;

   comm_open("/dev/tty.BT-ECU2-S-DevB");

   tcgetattr(0,&t);
   t.c_lflag&=~(ICANON|ECHO);
   tcsetattr(0,TCSANOW,&t);

   flags_stdio=fcntl(0, F_GETFL,0);
   flags_stdio|=O_NONBLOCK;
   fcntl(0, F_SETFL,flags_stdio);


   sequencer_init();
   sequencer_move_body(0,100,&startup_vector);

   wp.step_direction.x=0; 
   wp.step_direction.y=0;
   wp.step_direction.z=0;
   wp.step_duration=20;
   wp.step_height=20;

   sequencer_walk(&wp);
   printf("\n\nadws: body speed\n rf: body height \n 12: increase/decrease step duration \n 34: increase/decrease step height \nq: quit\n> ");

   while(!quit) {
      if(read(0,&c,1)==1) {
        switch(c) {
            case 'q':
            case  27:
               quit=1;
               break;
            case ' ': wp.step_direction.x=0;wp.step_direction.y=0; break;
            case 'w': wp.step_direction.x+=10; break;
            case 's': wp.step_direction.x-=10; break;
            case 'a': wp.step_direction.y+=10; break;
            case 'd': wp.step_direction.y-=10; break;
            case '1': if(wp.step_duration>1) wp.step_duration-=1; break;
            case '2': wp.step_duration+=1; break;
            case '3': if(wp.step_height>5) wp.step_height-=5; break;
            case '4': wp.step_height+=5; break;
            case 'r':
               v.x=0; v.y=0; v.z=-10;
               sequencer_move_body(0,10,&v);
               break;
            case 'f':
               v.x=0; v.y=0; v.z=10;
               sequencer_move_body(0,10,&v);
               break;

         }
         printf("X speed: %f Y speed: %f Step high: %f Step duration: %i \n",wp.step_direction.x, wp.step_direction.y, wp.step_height, wp.step_duration);
      }

      t_frame_start=get_time();
      sequencer_run_frame(&idle_position);
      ik(&idle_position);
      ik_to_servos(&idle_position);
      t_frame_end=get_time();
      if(t_frame_end-t_frame_start>20000) {
         printf("frame %d slack %d\n",frame,t_frame_end-t_frame_start);
      }
      while(t_frame_end<t_frame_start+20000) t_frame_end=get_time();
//      ik_to_servos(&idle_position);
//      sequencer_dump();
      frame++;
   }
   comm_close();

   flags_stdio=fcntl(0, F_GETFL,0); flags_stdio&=~(O_NONBLOCK); fcntl(0,
   F_SETFL,flags_stdio);


   tcgetattr(0,&t);
   t.c_lflag|=(ICANON|ECHO|ECHOE|ISIG);
   tcsetattr(0,TCSANOW,&t);

   return 0;
}

void
ik_to_servos(bot *b) {
   int i,j;

   for(i=0;i<NUM_LEGS;i++) {
      for(j=0;j<3;j++) {
         if(!isnan(b->leg[i].ik_angle[j])) {
            servo_angle[i*3+j]=b->leg[i].ik_angle[j]*180/M_PI;
         }
      }
   }
//   printf("update servos\n");
   comm_update_servos(1);
}


int
get_time() {
   struct timeval t;
   gettimeofday(&t,NULL);
   return (t.tv_sec*1000000+t.tv_usec);

}
